function [sc_all, conditions] = loadEckerData(minTrial)
    load('ecker_data_v1_binned_static.mat')
    evokedStartBin = 30;
    evokedEndBin = 70;
    
    sc_all = {};
    sc_all_binned = {};
    conditions = {};
    
    nSess = length(data);
    for sess = 1:nSess
        if size(data{sess}.spikes,4) < minTrial
            continue
        end
        
        [nUnit, nCond, nBin, nTrial] = size(data{sess}.spikes);

        % throwing out contaminated clusters
        nUnitNonCont = sum(data{sess}.contamination<0.05);
        dataNonCont = data{sess}.spikes(data{sess}.contamination<0.05,:,evokedStartBin:evokedEndBin,:);

        % throwing out cells with very low rates in either condition
        minCount = ceil((evokedEndBin - evokedStartBin) * nTrial / 2000);
        badRate = [];
        for c = 1:nCond
            evokedData = dataNonCont(:,c,:,:); % nUnitNC x 1 x nEvokedBin x nTrial
            d = squeeze(sum(evokedData,3)); % nUnit x nTrial
            badRate = union(badRate, find(sum(d,2) < minCount));
        end
        goodRate = setdiff(1:nUnitNonCont,badRate);
        dataGood = dataNonCont(goodRate,:,:,:);
        nUnitGood = length(goodRate);

        sc_all{end+1} = squeeze(sum(dataGood,3));  % nUnitGood x nCond x nTrial
        conditions{end+1} = data{sess}.conditions;
        sc_all_binned{end+1} = dataGood;
    end
end
