function poisson_integ_population(n_unit, corrbeta_P, corrbeta_RG, plotStuff, seed, recalc)
        
    setrandseed(seed);
    rerun = 5;

    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%% DATA %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    [all_spikes, conditions] = loadEckerData(38);

    data_unit_num = 0;
    for i = 1:length(all_spikes)
        data_unit_num = data_unit_num + size(all_spikes{i},1);
    end
    
    [ecker_sc_lc, ecker_sc_hc] = separateContrast(all_spikes, conditions);    
    [cEsc_lc,mEsc_lc,fEsc_lc] = getStats(ecker_sc_lc);
    [cEsc_hc,mEsc_hc,fEsc_hc] = getStats(ecker_sc_hc);          
    
    [mEsc_po,mEsc_no,fEsc_po,fEsc_no,cEsc_po,cEsc_no] = separateOrientation(all_spikes, conditions);
    
    [delta_std_ctr_E,delta_std_ori_E,delta_mean_ctr_E,delta_mean_ori_E] = histogramStability(false,all_spikes,conditions);
    
    e_dstd_ori = std(cEsc_po) - std(cEsc_no);
    e_dstd_ctr = std(cEsc_hc) - std(cEsc_lc);
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%% SIMULATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if recalc
        [cPsc_lc,cPsc_hc,cRGsc_lc,cRGsc_hc,cPsc_po,cPsc_no,cRGsc_po,cRGsc_no,corrmat_P,corrmat_RG,mff] = simulateCorrelations(n_unit, corrbeta_P, corrbeta_RG, 25);        
        delta_std_ctr_P = zeros(rerun,1);
        delta_std_ctr_RG = zeros(rerun,1);
        delta_std_ori_P = zeros(rerun,1);
        delta_std_ori_RG = zeros(rerun,1);
        delta_mean_ctr_P = zeros(rerun,1);
        delta_mean_ctr_RG = zeros(rerun,1);
        delta_mean_ori_P = zeros(rerun,1);
        delta_mean_ori_RG = zeros(rerun,1);
        for i = 1:rerun
            printCounter(i, 'maxVal', rerun, 'stringVal', 'Errorbar');
            [cPsc_lc_act,cPsc_hc_act,cRGsc_lc_act,cRGsc_hc_act,cPsc_po_act,cPsc_no_act,cRGsc_po_act,cRGsc_no_act,~,~] = simulateCorrelations(data_unit_num, corrbeta_P, corrbeta_RG, 1);

            delta_std_ctr_P(i) = std(cPsc_hc_act)-std(cPsc_lc_act);
            delta_std_ctr_RG(i) = std(cRGsc_hc_act)-std(cRGsc_lc_act);
            delta_std_ori_P(i) = std(cPsc_po_act)-std(cPsc_no_act);
            delta_std_ori_RG(i) = std(cRGsc_po_act)-std(cRGsc_no_act);

            delta_mean_ctr_P(i) = mean(cPsc_hc_act)-mean(cPsc_lc_act);
            delta_mean_ctr_RG(i) = mean(cRGsc_hc_act)-mean(cRGsc_lc_act);
            delta_mean_ori_P(i) = mean(cPsc_po_act)-mean(cPsc_no_act);
            delta_mean_ori_RG(i) = mean(cRGsc_po_act)-mean(cRGsc_no_act);
        end
        
        cbp = [0.5 1 2 5 10];
        mp_std = zeros(length(cbp), rerun);
        ori_dstd = zeros(length(cbp), rerun);
        ctr_dstd = zeros(length(cbp), rerun);
        for b = 1:length(cbp)        
            for r = 1:rerun
                printCounter(r, 'maxVal', rerun, 'stringVal', sprintf('Beta %.2f',cbp(b)));
                [cPsc_lc_act,cPsc_hc_act,~,~,cPsc_po_act,cPsc_no_act,~,~,corrmat_P_act,~] = simulateCorrelations(data_unit_num, cbp(b), 0, 1); % this could be 50                
                
                mp_std(b,r) = std(triuvals(corrmat_P_act));
                ori_dstd(b,r) = std(cPsc_po_act) - std(cPsc_no_act);
                ctr_dstd(b,r) = std(cPsc_hc_act) - std(cPsc_lc_act);            
            end
        end
        
        varianceSteps = 4;
        ff_p = zeros(varianceSteps, rerun);
        ori_dstd_varstep = zeros(varianceSteps, rerun);
        ctr_dstd_varstep = zeros(varianceSteps, rerun);
        for b = 1:varianceSteps
            act_input_var = 4 * 2^(2*(b-1));
            act_input_mean = 50 * 0.5^(2*(b-1));
            for r = 1:rerun
                printCounter(r, 'maxVal', rerun, 'stringVal', sprintf('Variance step %d',b));
                [cPsc_lc_act,cPsc_hc_act,~,~,cPsc_po_act,cPsc_no_act,~,~,~,~,mff_act] = simulateCorrelations(data_unit_num,corrbeta_P,0,1,'mp_var_b_P',act_input_var,'tc_coeff_P',act_input_mean);
                ff_p(b,r) = mean(mff_act.fPsc_hc);
                ori_dstd_varstep(b,r) = std(cPsc_po_act) - std(cPsc_no_act);
                ctr_dstd_varstep(b,r) = std(cPsc_hc_act) - std(cPsc_lc_act);            
            end
        end
        
        save('pop_save.mat','cPsc_lc','cPsc_hc','cRGsc_lc','cRGsc_hc','cPsc_po','cPsc_no','cRGsc_po','cRGsc_no','corrmat_P','corrmat_RG', ...
            'delta_std_ctr_P','delta_std_ctr_RG','delta_std_ori_P', 'delta_std_ori_RG','delta_mean_ctr_P','delta_mean_ctr_RG','delta_mean_ori_P','delta_mean_ori_RG', ...
            'mp_std','ori_dstd','ctr_dstd','mff','ff_p','ori_dstd_varstep','ctr_dstd_varstep');
    else
        load('pop_save.mat')
    end
    
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%% PLOTTING %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    fprintf('Data HC corr STD %.2f\n',std(cEsc_hc));
    fprintf('DS HC corr STD %.2f\n',std(cPsc_hc));
    fprintf('RG HC corr STD %.2f\n',std(cRGsc_hc));
    
    fprintf('Data NO corr STD %.2f\n',std(cEsc_no));
    fprintf('DS NO corr STD %.2f\n',std(cPsc_no));
    fprintf('RG NO corr STD %.2f\n',std(cRGsc_no));
    
    if plotStuff
        close all
        
        legends = {'HC', 'LC'};
        res = 40;
        rn = 5;
        cn = 3;
        
        figCtrSup = figure('units','normalized','outerposition',[0 0 1 1]);
        
        plotHistogramRow(rn,cn,[1 2 3],{mff.mu_hc_P mff.mu_lc_P; mff.var_hc_P mff.var_lc_P; triuvals(corrmat_P) []},'DS MP',{'mean','variance / FF','correlation'},legends, res)
        plotHistogramRow(rn,cn,[4 5 6],{mff.mu_hc_RG mff.mu_lc_RG; mff.var_hc_RG mff.var_lc_RG; triuvals(corrmat_RG) []},'RG MP',{},legends, res)
        plotHistogramRow(rn,cn,[7 8 9],{mff.mPsc_hc mff.mPsc_lc; mff.fPsc_hc mff.fPsc_lc; cPsc_hc cPsc_lc},'DS SC',{},legends, res)
        plotHistogramRow(rn,cn,[10 11 12],{mff.mRGsc_hc mff.mRGsc_lc; mff.fRGsc_hc mff.fRGsc_lc; cRGsc_hc cRGsc_lc},'RG SC',{},legends, res)
        plotHistogramRow(rn,cn,[13 14 15],{mEsc_hc mEsc_lc; fEsc_hc fEsc_lc; cEsc_hc cEsc_lc},'Data SC',{},legends, res)
    
        set(gcf,'PaperPositionMode','auto')
        print(figCtrSup,'pop_ctr_sup.png','-dpng','-r0');
        
        figCtr = figure('units','normalized','outerposition',[0 0 1 1]);
        plotHistogramRow(2,3,[1 2 3],{cPsc_hc cPsc_lc; cRGsc_hc cRGsc_lc; cEsc_hc cEsc_lc},'SC correlations',{'Doubly stochastic','Rectified Gaussian','Experiment'},{},res)
        equalizePlots(2,3,1,1,1,3,true,true)        
        subplot(2,3,3)
        legend(legends)
        inset(2,3,1,@() doubleHistPlot(triuvals(corrmat_P), [], res, {}, [], false),'MP corrs',[-1 1])
        inset(2,3,2,@() doubleHistPlot(triuvals(corrmat_RG), [], res, {}, [], false),'MP corrs',[-1 1])        
          
        subplot(2,4,5)
        barPlot({delta_mean_ctr_P, delta_mean_ctr_RG, delta_mean_ctr_E},{'DS','RG','E'},[],'\Delta mean (HC - LC)','','none')
        
        subplot(2,4,6)
        barPlot({delta_std_ctr_P, delta_std_ctr_RG, delta_std_ctr_E},{'DS','RG','E'},[],'\Delta STD (HC - LC)','','none')
        equalizePlots(2,4,2,2,1,2,false,true)
        
        subplot(2,4,7)
        hold on
        set(gca,'FontSize',16);
        %errorbar(mean(mp_std,2),mean(ctr_dstd,2),std(ctr_dstd,0,2) ./ sqrt(rerun),'LineWidth', 2)
        scatter(mp_std(:),ctr_dstd(:),'k','filled')        
        xl = [min(mp_std(:))-0.1 max(mp_std(:))+0.1];
        xlim(xl);
        plot(xl,[e_dstd_ctr e_dstd_ctr],'--r','LineWidth', 2)
        text(mp_std(end-1,1),e_dstd_ctr*0.9,'Experiment','FontSize',12)
        plot(xl,[0 0],':k','LineWidth', 1)
        xlim(xl)
        dstitle = title('\Delta STD (HC-LC) in the DS model','FontSize',16);
        origpos = get(dstitle,'Position');
        set(dstitle,'Position',[origpos(1) + 0.35, origpos(2) + 0.005, origpos(3)]);
        xlabel('MP correrlation STD','FontSize',16);
        ylabel('SC correrlation STD difference','FontSize',16);                  
        
        subplot(2,4,8)
        hold on
        set(gca,'FontSize',16);
        scatter(ff_p(:),ctr_dstd_varstep(:),'k','filled')
        xl = [min(ff_p(:))-0.5 max(ff_p(:))+0.5];
        xlim(xl);
        plot(xl,[e_dstd_ctr e_dstd_ctr],'--r','LineWidth', 2)
        text(ff_p(end-1,1),e_dstd_ctr*0.9,'Experiment','FontSize',12)
        plot(xl,[0 0],':k','LineWidth', 1)
        xlim(xl)
        xlabel('Mean SC Fano factor','FontSize',16);
        
        set(gcf,'PaperPositionMode','auto')
        print(figCtr,'pop_ctr_tc_data.png','-dpng','-r0');
                

        legends = {'PO', 'NO'};
        
        figOriSup = figure('units','normalized','outerposition',[0 0 1 1]);
        
        plotHistogramRow(rn,cn,[1 2 3],{mff.mu_po_P mff.mu_no_P; mff.var_po_P mff.var_no_P; triuvals(corrmat_P) []},'DS MP',{'mean','variance / FF','correlation'},legends, res)
        plotHistogramRow(rn,cn,[4 5 6],{mff.mu_po_RG mff.mu_no_RG; mff.var_po_RG mff.var_no_RG; triuvals(corrmat_RG) []},'RG MP',{},legends, res)
        plotHistogramRow(rn,cn,[7 8 9],{mff.mPsc_po mff.mPsc_no; mff.fPsc_po mff.fPsc_no; cPsc_po cPsc_no},'DS SC',{},legends, res)
        plotHistogramRow(rn,cn,[10 11 12],{mff.mRGsc_po mff.mRGsc_no; mff.fRGsc_po mff.fRGsc_no; cRGsc_po cRGsc_no},'RG SC',{},legends, res)
        plotHistogramRow(rn,cn,[13 14 15],{mEsc_po mEsc_no; fEsc_po fEsc_no; cEsc_po cEsc_no},'Data SC',{},legends, res)
    
        set(gcf,'PaperPositionMode','auto')
        print(figOriSup,'pop_ori_sup.png','-dpng','-r0');

        figOri = figure('units','normalized','outerposition',[0 0 1 1]);
        plotHistogramRow(2,3,[1 2 3],{cPsc_po cPsc_no; cRGsc_po cRGsc_no; cEsc_po cEsc_no},'SC correlations',{'Doubly stochastic','Rectified Gaussian','Experiment'},{},res)
        equalizePlots(2,3,1,1,1,3,true,true)
        subplot(2,3,3)
        legend(legends)
        inset(2,3,1,@() doubleHistPlot(triuvals(corrmat_P), [], res, {}, [], false),'MP corrs',[-1 1])
        inset(2,3,2,@() doubleHistPlot(triuvals(corrmat_RG), [], res, {}, [], false),'MP corrs',[-1 1])
        
        subplot(2,4,5)
        barPlot({delta_mean_ori_P, delta_mean_ori_RG, delta_mean_ori_E},{'DS','RG','E'},[],'\Delta mean (PO - NO)','','none')        
        
        subplot(2,4,6)
        barPlot({delta_std_ori_P, delta_std_ori_RG, delta_std_ori_E},{'DS','RG','E'},[],'\Delta STD (PO - NO)','','none')
        equalizePlots(2,4,2,2,1,2,false,true)
        
        subplot(2,4,7)
        hold on
        set(gca,'FontSize',16);        
        %errorbar(mean(mp_std,2),mean(ori_dstd,2),std(ori_dstd,0,2) ./ sqrt(rerun),'LineWidth', 2)
        scatter(mp_std(:),ori_dstd(:),'k','filled')
        xl = [min(mp_std(:))-0.1 max(mp_std(:))+0.1];
        xlim(xl);
        plot(xl,[e_dstd_ori e_dstd_ori],'--r','LineWidth', 2)
        text(mp_std(end-1,1),e_dstd_ori*0.95,'Experiment','FontSize',12)
        plot(xl,[0 0],':k','LineWidth', 1)
        xlim(xl)
        dstitle = title('\Delta STD (PO-NO) in the DS model','FontSize',16);
        origpos = get(dstitle,'Position');
        set(dstitle,'Position',[origpos(1) + 0.35, origpos(2) + 0.005, origpos(3)]);
        xlabel('MP correrlation STD','FontSize',16);
        ylabel('SC correrlation STD difference','FontSize',16);                  
                
        subplot(2,4,8)
        hold on
        set(gca,'FontSize',16);        
        scatter(ff_p(:),ori_dstd_varstep(:),'k','filled')
        xl = [min(ff_p(:))-0.5 max(ff_p(:))+0.5];
        xlim(xl);
        plot(xl,[e_dstd_ctr e_dstd_ctr],'--r','LineWidth', 2)
        text(ff_p(end-1,1),e_dstd_ctr*0.9,'Experiment','FontSize',12)
        plot(xl,[0 0],':k','LineWidth', 1)
        xlim(xl)
        xlabel('Mean SC Fano factor','FontSize',16);
        
        set(gcf,'PaperPositionMode','auto')
        print(figOri,'pop_ori_tc_data.png','-dpng','-r0');        
    end
end


function plotHistogramRow(rownum,colnum,plotindexes,data,label,titles,legends,resolution)
    % data has to be a colnum x 2 cell array
    for i = 1:length(plotindexes)        
        subplot(rownum,colnum,plotindexes(i))
        doubleHistPlot(data{i,1}, data{i,2}, resolution, legends, [], false)
        if i == 1
            ylabel(label)
        end
        if ~isempty(titles)
            title(titles{i})
        end
    end
end

function inset(rnum,cnum,spidx,call,text,limits)
    subplot(rnum,cnum,spidx)
    pos = get(gca,'Position');
    inset_width = pos(3) * 0.3 + 0.01;
    inset_height = pos(4) * 0.3 + 0.01;
    x_start = pos(1) + pos(3) - inset_width - 0.01;
    y_start = pos(2) + pos(4) - inset_height - 0.01;
    axes('Position',[x_start y_start inset_width inset_height])    
    call();
    set(gca,'XTickLabel',{},'box','on');
    xlabel(text,'FontSize',12)
    xlim(limits)
end

function setrandseed(randseed)
    if strcmp(randseed,'last')
        % TODO check if exsits
        load('lastrandseed');
    elseif strcmp(randseed,'leave')
        return
    end
    s = RandStream('mt19937ar','Seed',randseed);
    RandStream.setGlobalStream(s);
    randseed = s.Seed;
    save('lastrandseed.mat','randseed');
end