function [cPsc_lc,cPsc_hc,cRGsc_lc,cRGsc_hc,cPsc_po,cPsc_no,cRGsc_po,cRGsc_no,corrmat_P,corrmat_RG,mff] = simulateCorrelations(n_unit, corrbeta_P, corrbeta_RG, n_bin, varargin)
    parser = inputParser;
    addParameter(parser,'mp_var_b_P',4,@isnumeric);
    addParameter(parser,'mp_var_b_RG',4,@isnumeric);
    addParameter(parser,'tc_coeff_P',50,@isnumeric);
    addParameter(parser,'tc_coeff_RG',100,@isnumeric);
    parse(parser,varargin{:});
    params = parser.Results;

    base_rate = 20; % Hz
    exponent = 1.4;
    theta = 0;       
    n_trial = 1000;    
    bin_length = 20; % ms    
    k = base_rate / (1000 / bin_length); % base number of spikes within a bin
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%% PARAMETERS %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    contrast_mean_factor_P = 2;
    contrast_var_factor_P = 0.5;        
    
    contrast_mean_factor_RG = 2;
    contrast_var_factor_RG = 0.5;
    
    orient_range = 180;
    preferred_orientations = orient_range * rand([n_unit 1]);
    tuning_width = orient_range * 0.2;    
    preference_threshold = orient_range * 0.5;
    
    % stimulus_orientation = orient_range/2 * rand()
    stimulus_orientation = 5;
    
    pref_units = abs(preferred_orientations - stimulus_orientation) < preference_threshold;
    nonpref_units = abs(preferred_orientations - stimulus_orientation) >= preference_threshold; 
 
    mp_mean_mean_P = params.tc_coeff_P * max( normpdf(ones(n_unit,1) * stimulus_orientation, preferred_orientations, ones(n_unit,1) * tuning_width), ...
        normpdf(ones(n_unit,1) * (stimulus_orientation + orient_range/2) , preferred_orientations, ones(n_unit,1) * tuning_width));
    mp_mean_mean_RG = params.tc_coeff_RG * max( normpdf(ones(n_unit,1) * stimulus_orientation, preferred_orientations, ones(n_unit,1) * tuning_width), ...
        normpdf(ones(n_unit,1) * (stimulus_orientation + orient_range/2) , preferred_orientations, ones(n_unit,1) * tuning_width));
    
    mp_mean_var_P = 0.01;
    mp_mean_var_RG = 0.01;
    
    mp_var_a_P = 3;
    mp_var_a_RG = 3;    
            
    
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%% SIMULATION %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    corrmat_P = vineBeta(n_unit, corrbeta_P);
    
    mu_lc_P = mp_mean_mean_P + mp_mean_var_P * randn([n_unit 1]);
    var_lc_P = inverse_gamma(mp_var_a_P, params.mp_var_b_P, n_unit);
    C_lc_P = diag(sqrt(var_lc_P)) * corrmat_P * diag(sqrt(var_lc_P));    
    
    mu_hc_P = mp_mean_mean_P * contrast_mean_factor_P + mp_mean_var_P * randn([n_unit 1]);
    var_hc_P = inverse_gamma(mp_var_a_P, params.mp_var_b_P * contrast_var_factor_P, n_unit);
    C_hc_P = diag(sqrt(var_hc_P)) * corrmat_P * diag(sqrt(var_hc_P));                    
    
    rates_P_lc = gen_rates(n_trial, n_bin, mu_lc_P, C_lc_P, theta, exponent, k);
    rates_P_hc = gen_rates(n_trial, n_bin, mu_hc_P, C_hc_P, theta, exponent, k);
    samples_P_lc = spikes_poisson(rates_P_lc);
    samples_P_hc = spikes_poisson(rates_P_hc);
    [cPsc_lc,mPsc_lc,fPsc_lc] = getStats(samples_P_lc);
    [cPsc_hc,mPsc_hc,fPsc_hc] = getStats(samples_P_hc);
    spikes_hc_po_P = samples_P_hc(pref_units, :);
    spikes_hc_no_P = samples_P_hc(nonpref_units, :);
    [cPsc_po,mPsc_po,fPsc_po] = getStats(spikes_hc_po_P);
    [cPsc_no,mPsc_no,fPsc_no] = getStats(spikes_hc_no_P);
    
    if corrbeta_RG ~= 0
        corrmat_RG = vineBeta(n_unit, corrbeta_RG);
        
        mu_lc_RG = mp_mean_mean_RG + mp_mean_var_RG * randn([n_unit 1]);
        var_lc_RG = inverse_gamma(mp_var_a_RG, params.mp_var_b_RG, n_unit);
        C_lc_RG = diag(sqrt(var_lc_RG)) * corrmat_RG * diag(sqrt(var_lc_RG));

        mu_hc_RG = mp_mean_mean_RG * contrast_mean_factor_RG + mp_mean_var_RG * randn([n_unit 1]);
        var_hc_RG = inverse_gamma(mp_var_a_RG, params.mp_var_b_RG * contrast_var_factor_RG, n_unit);
        C_hc_RG = diag(sqrt(var_hc_RG)) * corrmat_RG * diag(sqrt(var_hc_RG));
    
        rates_RG_lc = gen_rates(n_trial, n_bin, mu_lc_RG, C_lc_RG, theta, exponent, k);
        rates_RG_hc = gen_rates(n_trial, n_bin, mu_hc_RG, C_hc_RG, theta, exponent, k);    
        samples_RG_lc = spikes_determ(rates_RG_lc);    
        samples_RG_hc = spikes_determ(rates_RG_hc);        
        [cRGsc_lc,mRGsc_lc,fRGsc_lc] = getStats(samples_RG_lc);
        [cRGsc_hc,mRGsc_hc,fRGsc_hc] = getStats(samples_RG_hc);                   
        spikes_hc_po_RG = samples_RG_hc(pref_units, :);
        spikes_hc_no_RG = samples_RG_hc(nonpref_units, :);        
        [cRGsc_po,mRGsc_po,fRGsc_po] = getStats(spikes_hc_po_RG);
        [cRGsc_no,mRGsc_no,fRGsc_no] = getStats(spikes_hc_no_RG);
    else
        cRGsc_lc = [];
        cRGsc_hc = [];
        cRGsc_po = [];
        cRGsc_no = [];
        corrmat_RG = [];        
    end
    
    if nargout > 10
        mff.mu_lc_P = mu_lc_P;
        mff.mu_hc_P = mu_hc_P;        
        
        mff.var_lc_P = var_lc_P;
        mff.var_hc_P = var_hc_P;        
        
        mff.mu_po_P = mu_hc_P(pref_units);
        mff.mu_no_P = mu_hc_P(nonpref_units);
        
        mff.var_po_P = var_hc_P(pref_units);
        mff.var_no_P = var_hc_P(nonpref_units);
        
        mff.mPsc_lc = mPsc_lc;
        mff.mPsc_hc = mPsc_hc;
        mff.mPsc_po = mPsc_po;
        mff.mPsc_no = mPsc_no;
        
        mff.fPsc_lc = fPsc_lc;
        mff.fPsc_hc = fPsc_hc;
        mff.fPsc_po = fPsc_po;
        mff.fPsc_no = fPsc_no;
        
        if corrbeta_RG ~= 0
            mff.mu_lc_RG = mu_lc_RG;
            mff.mu_hc_RG = mu_hc_RG;

            mff.var_lc_RG = var_lc_RG;
            mff.var_hc_RG = var_hc_RG;        

            mff.mu_po_RG = mu_hc_RG(pref_units);
            mff.mu_no_RG = mu_hc_RG(nonpref_units);

            mff.var_po_RG = var_hc_RG(pref_units);
            mff.var_no_RG = var_hc_RG(nonpref_units);                

            mff.mRGsc_lc = mRGsc_lc;
            mff.mRGsc_hc = mRGsc_hc;
            mff.mRGsc_po = mRGsc_po;
            mff.mRGsc_no = mRGsc_no;       

            mff.fRGsc_lc = fRGsc_lc;
            mff.fRGsc_hc = fRGsc_hc;
            mff.fRGsc_po = fRGsc_po;
            mff.fRGsc_no = fRGsc_no;
        end
    end
end

function S = vineBeta(d, betaparam)
    % http://stats.stackexchange.com/questions/124538/how-to-generate-a-large-full-rank-random-correlation-matrix-with-some-strong-cor
    
    P = zeros(d);           %// storing partial correlations
    S = eye(d);

    for k = 1:d-1
        for i = k+1:d
            P(k,i) = betarnd(betaparam,betaparam); %// sampling from beta
            P(k,i) = (P(k,i)-0.5)*2;     %// linearly shifting to [-1, 1]
            p = P(k,i);
            for l = (k-1):-1:1 %// converting partial correlation to raw correlation
                p = p * sqrt((1-P(l,i)^2)*(1-P(l,k)^2)) + P(l,i)*P(l,k);
            end
            S(k,i) = p;
            S(i,k) = p;
        end
    end

    %// permuting the variables to make the distribution permutation-invariant
    permutation = randperm(d);
    S = S(permutation, permutation);
end

function y = inverse_gamma(a, b, n)
    x=gamrnd(a, 1/b, [n 1]);
    y=1./x;
end

function r = gen_rates(n_trial, n_bin, mu, C, Vth, exponent, k)
    n_unit = length(mu);
    u = zeros(n_unit, n_trial, n_bin);
    for t = 1:n_trial
        u(:,t,:) = mvnrnd(repmat(mu',[n_bin 1]), C)';
    end
    uRect = u - Vth; 
    uRect(uRect<0) = 0;
    r = k * uRect.^exponent;
end

function s = spikes_poisson(r)
    s = poissrnd(r); % n_unit x n_trial x n_bin
    s = squeeze(sum(s,3)); % n_unit x n_trial
end

function s = spikes_determ(r)
    s = floor(r);
    s = squeeze(sum(s,3)); % n_unit x n_trial
end
